# Script Optimize.py
# Copyright (c) 2006-2006 Quadralay Corporation
#


import os
import re

import Books
import Filter


def PruneDirectory(ParamPath):
  VarNames = os.listdir(ParamPath)
  if (len(VarNames) == 0):
    os.rmdir(ParamPath)
    PruneDirectory(os.path.dirname(ParamPath))


def DeleteFile(ParamPath):
  # Delete the file
  #
  os.unlink(ParamPath)

  # Clean up empty directories
  #
  PruneDirectory(os.path.dirname(ParamPath))


def DeleteDirectory(ParamPath):
  VarNames = os.listdir(ParamPath)
  for VarName in VarNames:
    VarAbsoluteName = os.path.join(ParamPath, VarName)
    if (os.path.isdir(VarAbsoluteName)):
      DeleteDirectory(VarAbsoluteName)
    else:
      os.unlink(VarAbsoluteName)

  os.rmdir(ParamPath)


def LoadLines(ParamPath):
  VarFile = open(ParamPath, 'r')
  VarLines = VarFile.readlines()
  VarFile.close()

  return VarLines


def CombineFileList(ParamPath,
                    ParamFileList):
  # Open file
  #
  VarFile = open(ParamPath, 'w')

  # Combine files
  #
  for (VarFilePath, VarFilter) in ParamFileList:
    VarLines = LoadLines(VarFilePath)
    VarFilteredLines = VarFilter(VarLines)
    VarFile.writelines(VarFilteredLines)

  # Close combined script
  #
  VarFile.close()


def ApplyFileList(ParamFileList):
  # Combine and filter scripts
  #
  for (VarFilePath, VarFilter) in ParamFileList:
    # Load file
    #
    VarLines = LoadLines(VarFilePath)

    # Filter
    #
    VarFilteredLines = VarFilter(VarLines)

    # Rewrite file
    #
    VarFile = open(VarFilePath, 'w')
    VarFile.writelines(VarFilteredLines)
    VarFile.close()


def DeleteFileList(ParamFileList):
  # Record unique names
  #
  VarFilePaths = {}
  for (VarFilePath, VarFilter) in ParamFileList:
    VarFilePaths[VarFilePath] = True

  # Delete unique file paths
  #
  for VarFilePath in VarFilePaths:
    if (os.path.exists(VarFilePath)):
      DeleteFile(VarFilePath)


def CombineCommonPrivate(ParamPath):
  VarScriptsDirectoryPath = os.path.join(ParamPath, 'wwhelp', 'wwhimpl', 'common', 'private')

  VarCombinedScriptPath = VarScriptsDirectoryPath + '.js'
  VarFileList = [
                  (os.path.join(VarScriptsDirectoryPath, 'books.js'), Filter.FilterPassThrough),
                  (os.path.join(VarScriptsDirectoryPath, 'locale.js'), Filter.FilterPassThrough),
                  (os.path.join(VarScriptsDirectoryPath, 'options.js'), Filter.FilterPassThrough),
                  (os.path.join(VarScriptsDirectoryPath, 'popupf.js'), Filter.FilterPassThrough),
                 ]

  CombineFileList(VarCombinedScriptPath, VarFileList)
  DeleteFileList(VarFileList)


def CombineCommonScripts(ParamPath):
  VarScriptsDirectoryPath = os.path.join(ParamPath, 'wwhelp', 'wwhimpl', 'common', 'scripts')

  VarCombinedScriptPath = VarScriptsDirectoryPath + '.js'
  VarFileList = [
                  (os.path.join(VarScriptsDirectoryPath, 'bookgrps.js'), Filter.FilterPassThrough),
                  (os.path.join(VarScriptsDirectoryPath, 'booklist.js'), Filter.FilterCommonBookListScript),
                  (os.path.join(VarScriptsDirectoryPath, 'browser.js'), Filter.FilterPassThrough),
                  (os.path.join(VarScriptsDirectoryPath, 'controls.js'), Filter.FilterPassThrough),
                  (os.path.join(VarScriptsDirectoryPath, 'filelist.js'), Filter.FilterPassThrough),
                  (os.path.join(VarScriptsDirectoryPath, 'handler.js'), Filter.FilterPassThrough),
                  (os.path.join(VarScriptsDirectoryPath, 'help.js'), Filter.FilterCommonHelpScript),
                  (os.path.join(VarScriptsDirectoryPath, 'highlt.js'), Filter.FilterPassThrough),
                  (os.path.join(VarScriptsDirectoryPath, 'popup.js'), Filter.FilterPassThrough),
                  (os.path.join(VarScriptsDirectoryPath, 'related.js'), Filter.FilterPassThrough),
                  (os.path.join(VarScriptsDirectoryPath, 'strutils.js'), Filter.FilterPassThrough),
                  (os.path.join(VarScriptsDirectoryPath, 'switch.js'), Filter.FilterPassThrough),
                ]

  CombineFileList(VarCombinedScriptPath, VarFileList)
  DeleteFileList(VarFileList)


def CombineJavaScriptPrivate(ParamPath):
  VarScriptsDirectoryPath = os.path.join(ParamPath, 'wwhelp', 'wwhimpl', 'js', 'private')

  VarCombinedScriptPath = VarScriptsDirectoryPath + '.js'
  VarFileList = [
                  (os.path.join(VarScriptsDirectoryPath, 'locale.js'), Filter.FilterPassThrough),
                  (os.path.join(VarScriptsDirectoryPath, 'options.js'), Filter.FilterPassThrough),
                ]

  CombineFileList(VarCombinedScriptPath, VarFileList)
  DeleteFileList(VarFileList)


def CombineJavaScriptScripts(ParamPath):
  VarScriptsDirectoryPath = os.path.join(ParamPath, 'wwhelp', 'wwhimpl', 'js', 'scripts')

  VarCombinedScriptPath = VarScriptsDirectoryPath + '.js'
  VarFileList = [
                  (os.path.join(VarScriptsDirectoryPath, 'favorite.js'), Filter.FilterPassThrough),
                  (os.path.join(VarScriptsDirectoryPath, 'handler.js'), Filter.FilterJavaScriptHandlerScript),
                  (os.path.join(VarScriptsDirectoryPath, 'index.js'), Filter.FilterJavaScriptIndexScript),
                  (os.path.join(VarScriptsDirectoryPath, 'javascpt.js'), Filter.FilterPassThrough),
                  (os.path.join(VarScriptsDirectoryPath, 'mltphrs.js'), Filter.FilterPassThrough),
                  (os.path.join(VarScriptsDirectoryPath, 'outlfast.js'), Filter.FilterPassThrough),
                  (os.path.join(VarScriptsDirectoryPath, 'outline.js'), Filter.FilterJavaScriptOutlineScript),
                  (os.path.join(VarScriptsDirectoryPath, 'outlsafe.js'), Filter.FilterPassThrough),
                  (os.path.join(VarScriptsDirectoryPath, 'pairs.js'), Filter.FilterPassThrough),
                  (os.path.join(VarScriptsDirectoryPath, 'panels.js'), Filter.FilterJavaScriptPanelsScript),
                  (os.path.join(VarScriptsDirectoryPath, 'phrase.js'), Filter.FilterPassThrough),
                  (os.path.join(VarScriptsDirectoryPath, 'search.js'), Filter.FilterJavaScriptSearchScript),
                  (os.path.join(VarScriptsDirectoryPath, 'tabs.js'), Filter.FilterPassThrough),
                ]

  CombineFileList(VarCombinedScriptPath, VarFileList)
  DeleteFileList(VarFileList)


def UpdateHTMLFiles(ParamPath):
  VarCommonHTMLDirectoryPath = os.path.join(ParamPath, 'wwhelp', 'wwhimpl', 'common', 'html')
  VarJavaScriptHTMLDirectoryPath = os.path.join(ParamPath, 'wwhelp', 'wwhimpl', 'js', 'html')

  VarFileList = [
                  (os.path.join(VarCommonHTMLDirectoryPath, 'content.htm'), Filter.FilterCommonHTMLFile),
                  (os.path.join(VarCommonHTMLDirectoryPath, 'frameset.htm'), Filter.FilterCommonHTMLFile),
                  (os.path.join(VarCommonHTMLDirectoryPath, 'init1.htm'), Filter.FilterCommonHTMLFile),
                  (os.path.join(VarCommonHTMLDirectoryPath, 'pagenav.htm'), Filter.FilterCommonHTMLFile),
                  (os.path.join(VarCommonHTMLDirectoryPath, 'reset.htm'), Filter.FilterCommonHTMLFile),
                  (os.path.join(VarCommonHTMLDirectoryPath, 'switch.htm'), Filter.FilterCommonHTMLFile),
                  (os.path.join(VarCommonHTMLDirectoryPath, 'wwhelp.htm'), Filter.FilterCommonHTMLFile),
                  (os.path.join(VarJavaScriptHTMLDirectoryPath, 'navigate.htm'), Filter.FilterJavaScriptHTMLFile),
                  (os.path.join(VarJavaScriptHTMLDirectoryPath, 'panelfnf.htm'), Filter.FilterJavaScriptHTMLFile),
                  (os.path.join(VarJavaScriptHTMLDirectoryPath, 'panelfni.htm'), Filter.FilterJavaScriptHTMLFile),
                  (os.path.join(VarJavaScriptHTMLDirectoryPath, 'panelfns.htm'), Filter.FilterJavaScriptHTMLFile),
                  (os.path.join(VarJavaScriptHTMLDirectoryPath, 'panelfsf.htm'), Filter.FilterJavaScriptHTMLFile),
                  (os.path.join(VarJavaScriptHTMLDirectoryPath, 'panelfsi.htm'), Filter.FilterJavaScriptHTMLFile),
                  (os.path.join(VarJavaScriptHTMLDirectoryPath, 'panelfss.htm'), Filter.FilterJavaScriptHTMLFile),
                  (os.path.join(VarJavaScriptHTMLDirectoryPath, 'tabs.htm'), Filter.FilterJavaScriptHTMLFile),
                  (os.path.join(VarJavaScriptHTMLDirectoryPath, 'wwhelp.htm'), Filter.FilterJavaScriptHTMLFile),
                ]

  ApplyFileList(VarFileList)


def UpdateWWHDataFiles(ParamPath,
                       ParamBookDirectoryPaths):
  # Build file map
  #
  if (len(ParamBookDirectoryPaths) > 1):
    VarFileList = []
    for VarBookDirectoryPath in ParamBookDirectoryPaths:
      VarAbsoluteBookDirectoryPath = os.path.normpath(os.path.join(ParamPath, VarBookDirectoryPath))

      VarFileList.append((os.path.join(VarAbsoluteBookDirectoryPath, 'wwhdata', 'common', 'towwhdir.js'), Filter.FilterToWWHDir))

    ApplyFileList(VarFileList)


def CombineBookData(ParamPath,
                    ParamBookDirectoryPaths):
  # Insure directory exists
  #
  VarWWHBookDataDirectoryPath = os.path.join(ParamPath, 'wwhbdata')
  if (not(os.path.exists(VarWWHBookDataDirectoryPath))):
    os.makedirs(VarWWHBookDataDirectoryPath)

  # Build file map
  #
  VarFileList = []
  VarBookIndex = 1
  for VarBookDirectoryPath in ParamBookDirectoryPaths:
    VarAbsoluteBookDirectoryPath = os.path.normpath(os.path.join(ParamPath, VarBookDirectoryPath))
    VarFilterBookData = Filter.FilterBookData(VarBookIndex)

    VarFileList.append((os.path.join(VarAbsoluteBookDirectoryPath, 'wwhdata', 'common', 'title.js'), VarFilterBookData))
    VarFileList.append((os.path.join(VarAbsoluteBookDirectoryPath, 'wwhdata', 'common', 'context.js'), VarFilterBookData))
    VarFileList.append((os.path.join(VarAbsoluteBookDirectoryPath, 'wwhdata', 'common', 'files.js'), VarFilterBookData))
    VarFileList.append((os.path.join(VarAbsoluteBookDirectoryPath, 'wwhdata', 'common', 'alinks.js'), VarFilterBookData))
    VarFileList.append((os.path.join(ParamPath, 'wwhelp', 'wwhimpl', 'common', 'scripts', 'bklist1s.js'), VarFilterBookData))

    VarBookIndex += 1

  # Combine book data files
  #
  VarCombinedScriptPath = os.path.join(VarWWHBookDataDirectoryPath, 'common.js')
  CombineFileList(VarCombinedScriptPath, VarFileList)

  # Delete unnecessary files
  #
  VarFileList = []
  VarBookIndex = 1
  for VarBookDirectoryPath in ParamBookDirectoryPaths:
    VarAbsoluteBookDirectoryPath = os.path.normpath(os.path.join(ParamPath, VarBookDirectoryPath))
    VarFilterBookData = Filter.FilterBookData(VarBookIndex)

    VarFileList.append((os.path.join(VarAbsoluteBookDirectoryPath, 'wwhdata', 'common', 'title.js'), VarFilterBookData))
    VarFileList.append((os.path.join(VarAbsoluteBookDirectoryPath, 'wwhdata', 'common', 'files.js'), VarFilterBookData))
    VarFileList.append((os.path.join(VarAbsoluteBookDirectoryPath, 'wwhdata', 'common', 'alinks.js'), VarFilterBookData))
    VarFileList.append((os.path.join(ParamPath, 'wwhelp', 'wwhimpl', 'common', 'scripts', 'bklist1s.js'), VarFilterBookData))

    VarBookIndex += 1

  DeleteFileList(VarFileList)


def CombineTOCBookData(ParamPath,
                       ParamBookDirectoryPaths):
  # Insure directory exists
  #
  VarWWHBookDataJavaScriptDirectoryPath = os.path.join(ParamPath, 'wwhbdata', 'js')
  if (not(os.path.exists(VarWWHBookDataJavaScriptDirectoryPath))):
    os.makedirs(VarWWHBookDataJavaScriptDirectoryPath)

  # Build file map
  #
  VarFileList = []
  VarBookIndex = 1
  for VarBookDirectoryPath in ParamBookDirectoryPaths:
    VarAbsoluteBookDirectoryPath = os.path.normpath(os.path.join(ParamPath, VarBookDirectoryPath))
    VarFilterTOCBookData = Filter.FilterTOCBookData(VarBookIndex)

    VarFileList.append((os.path.join(VarAbsoluteBookDirectoryPath, 'wwhdata', 'js', 'toc.js'), VarFilterTOCBookData))
    VarFileList.append((os.path.join(ParamPath, 'wwhelp', 'wwhimpl', 'js', 'scripts', 'outlin1s.js'), VarFilterTOCBookData))

    VarBookIndex += 1

  # Combine TOC files
  #
  VarCombinedScriptPath = os.path.join(VarWWHBookDataJavaScriptDirectoryPath, 'toc.js')
  CombineFileList(VarCombinedScriptPath, VarFileList)
  DeleteFileList(VarFileList)


def CombineIndexBookData(ParamPath,
                         ParamBookDirectoryPaths):
  # Insure directory exists
  #
  VarWWHBookDataJavaScriptDirectoryPath = os.path.join(ParamPath, 'wwhbdata', 'js')
  if (not(os.path.exists(VarWWHBookDataJavaScriptDirectoryPath))):
    os.makedirs(VarWWHBookDataJavaScriptDirectoryPath)

  # Build file map
  #
  VarFileList = []
  VarBookIndex = 1
  for VarBookDirectoryPath in ParamBookDirectoryPaths:
    VarAbsoluteBookDirectoryPath = os.path.normpath(os.path.join(ParamPath, VarBookDirectoryPath))
    VarFilterIndexBookData = Filter.FilterIndexBookData(VarBookIndex)

    VarFileList.append((os.path.join(VarAbsoluteBookDirectoryPath, 'wwhdata', 'js', 'index.js'), VarFilterIndexBookData))
    VarFileList.append((os.path.join(ParamPath, 'wwhelp', 'wwhimpl', 'js', 'scripts', 'index1s.js'), VarFilterIndexBookData))

    VarBookIndex += 1

  # Combine index files
  #
  VarCombinedScriptPath = os.path.join(VarWWHBookDataJavaScriptDirectoryPath, 'index.js')
  CombineFileList(VarCombinedScriptPath, VarFileList)
  DeleteFileList(VarFileList)


def CombineSearchBookData(ParamPath,
                          ParamBookDirectoryPaths):
  # Insure directory exists
  #
  VarWWHBookDataJavaScriptDirectoryPath = os.path.join(ParamPath, 'wwhbdata', 'js')
  if (not(os.path.exists(VarWWHBookDataJavaScriptDirectoryPath))):
    os.makedirs(VarWWHBookDataJavaScriptDirectoryPath)

  # Build file map
  #
  VarFileList = []
  VarBookIndex = 1
  for VarBookDirectoryPath in ParamBookDirectoryPaths:
    VarAbsoluteBookDirectoryPath = os.path.normpath(os.path.join(ParamPath, VarBookDirectoryPath))
    VarFilterSearchBookData = Filter.FilterSearchBookData(VarBookIndex)

    VarFileList.append((os.path.join(VarAbsoluteBookDirectoryPath, 'wwhdata', 'js', 'search.js'), VarFilterSearchBookData))
    VarFileList.append((os.path.join(ParamPath, 'wwhelp', 'wwhimpl', 'js', 'scripts', 'search1s.js'), VarFilterSearchBookData))

    VarBookIndex += 1

  # Combine search files
  #
  VarCombinedScriptPath = os.path.join(VarWWHBookDataJavaScriptDirectoryPath, 'search.js')
  CombineFileList(VarCombinedScriptPath, VarFileList)
  DeleteFileList(VarFileList)


def CombineSearchFiles(ParamPath,
                       ParamBookDirectoryPaths):
  # Define regular expression for search data files
  #
  VarReturnIntegerExpression = re.compile('^ *return +([0-9]+);$')

  # Process each book's search data
  #
  VarBookIndex = 1
  for VarBookDirectoryPath in ParamBookDirectoryPaths:
    VarAbsoluteBookDirectoryPath = os.path.normpath(os.path.join(ParamPath, VarBookDirectoryPath))
    VarSearchFilesDirectoryPath = os.path.join(VarAbsoluteBookDirectoryPath, 'wwhdata', 'js', 'search')

    # Determine number of search files
    #
    VarLines = LoadLines(VarSearchFilesDirectoryPath + '.js')
    VarInFunction = ''
    VarSearchFileCount = 0
    for VarLine in VarLines:
      if (VarInFunction == 'WWHBookData_SearchFileCount'):
        if (VarLine.startswith('}')):
          VarInFunction = ''
        else:
          VarMatch = VarReturnIntegerExpression.match(VarLine)
          if (VarMatch):
            VarSearchFileCount = int(VarMatch.group(1))
      elif (VarLine.startswith('function  WWHBookData_SearchFileCount()')):
        VarInFunction = 'WWHBookData_SearchFileCount'

    # Create file list
    #
    VarFileList = []
    for VarIndex in xrange(VarSearchFileCount):
      VarFileList.append((os.path.join(VarSearchFilesDirectoryPath, 'search%d.js' % (VarIndex)), Filter.FilterSearchFiles))

    # Combine search file data
    #
    VarCombinedDataPath = os.path.join(VarSearchFilesDirectoryPath, 'search.js')
    CombineFileList(VarCombinedDataPath, VarFileList)
    DeleteFileList(VarFileList)

    # Create single search file
    #
    VarSingleSearchFilePath = os.path.join(VarSearchFilesDirectoryPath, 'search0.js')
    VarSingleSearchFile = open(VarSingleSearchFilePath, 'w')
    VarLines = LoadLines(VarCombinedDataPath)
    DeleteFile(VarCombinedDataPath)
    VarSingleSearchFile.write('function BookData_Search(x,y)\n')
    VarSingleSearchFile.write('{\n')
    VarSingleSearchFile.writelines(VarLines)
    VarSingleSearchFile.write('}\n')
    VarSingleSearchFile.close()

    VarBookIndex += 1


def DeleteWWHelpXMLFiles(ParamPath):
  DeleteFile(os.path.join(ParamPath, 'wwhelp', 'books.xml'))
  DeleteFile(os.path.join(ParamPath, 'wwhelp', 'messages.xml'))
  DeleteFile(os.path.join(ParamPath, 'wwhelp', 'settings.xml'))


def DeleteWWHDataXMLFiles(ParamPath,
                          ParamBookDirectoryPaths):
  for VarBookDirectoryPath in ParamBookDirectoryPaths:
    VarAbsoluteBookDirectoryPath = os.path.normpath(os.path.join(ParamPath, VarBookDirectoryPath))
    VarWWHDataDirectoryPath = os.path.join(VarAbsoluteBookDirectoryPath, 'wwhdata')

    DeleteFile(os.path.join(VarWWHDataDirectoryPath, 'common', 'synonyms.xml'))
    DeleteFile(os.path.join(VarWWHDataDirectoryPath, 'xml', 'files.xml'))
    DeleteFile(os.path.join(VarWWHDataDirectoryPath, 'xml', 'ix.xml'))
    DeleteFile(os.path.join(VarWWHDataDirectoryPath, 'xml', 'toc.xml'))


def DeleteWWHelpDirectories(ParamPath,
                            ParamBookDirectoryPaths):
  if (len(ParamBookDirectoryPaths) > 1):
    for VarBookDirectoryPath in ParamBookDirectoryPaths:
      VarAbsoluteBookDirectoryPath = os.path.normpath(os.path.join(ParamPath, VarBookDirectoryPath))
      VarWWHelpDirectoryPath = os.path.join(VarAbsoluteBookDirectoryPath, 'wwhelp')

      DeleteDirectory(VarWWHelpDirectoryPath)


def Optimize(ParamPath):
  # Load book directory info
  #
  VarBookDirectoryPaths = Books.GetBookDirectoryPaths(ParamPath)

  # Update run-time files
  #
  CombineCommonPrivate(ParamPath)
  CombineCommonScripts(ParamPath)
  CombineJavaScriptPrivate(ParamPath)
  CombineJavaScriptScripts(ParamPath)
  UpdateHTMLFiles(ParamPath)
  UpdateWWHDataFiles(ParamPath, VarBookDirectoryPaths)

  # Update data files
  #
  CombineBookData(ParamPath, VarBookDirectoryPaths)
  CombineTOCBookData(ParamPath, VarBookDirectoryPaths)
  CombineIndexBookData(ParamPath, VarBookDirectoryPaths)
  CombineSearchFiles(ParamPath, VarBookDirectoryPaths)
  CombineSearchBookData(ParamPath, VarBookDirectoryPaths)

  # Clean up XML files
  #
  DeleteWWHelpXMLFiles(ParamPath)
  DeleteWWHDataXMLFiles(ParamPath, VarBookDirectoryPaths)
  DeleteWWHelpDirectories(ParamPath, VarBookDirectoryPaths)
